#include <system/MessageTransceiverThread.h>

class GameWorkerSession;

class GameSession : public muscle::Thread {
	public:
		GameSession(const char *server, int port, const char *user_name);
		virtual ~GameSession();
		
		void ChangeName(const char *new_user_name);
		
		virtual void MessageReceivedFromPartner(muscle::MessageRef msg);
		virtual void MessageReceivedFromNetwork(muscle::MessageRef msg);
		
		void SendMessageToPartner(muscle::MessageRef);
		void SendMessageToServer(muscle::MessageRef);
		
		virtual bool AllowConnectFrom(const char *user_name, int32 id);
		bool ConnectTo(const char *user);
		bool ConnectTo(int32 user_id);
		
		virtual void PartnerGone(); //-----Partner has disappeared
		void EndGame(); //---Disappear yourself
		
		virtual void NewUserLoggedOn(int32 id, const char *name);
		virtual void UserLoggedOff(int32 id, const char *name);
		
		virtual void ServerDisconnected(); //----The *server* has died, this prompts immediate suicide
		virtual void ServerConnectDone(); //-----Connected to *server*
		//---Please note that when AsyncConnectCompleted is called, we do *not*
		//		know our user id.
		
	protected:
		virtual const char * GetTypeName() const = 0; //---The name of your game
	
	private:
		virtual status_t MessageReceivedFromOwner(muscle::MessageRef msgRef, uint32 numLeft);
		void parse_out_uid_uname(const char *key, int32 *uid, const char **uname);
		
		int32 return_uid, connected_to;
		muscle::MessageTransceiverThread *mtt;
		GameWorkerSession *worker;
		friend class GameWorkerSession;
		
		thread_id connect_thread_waiting;
		volatile bool connect_result;
		muscle::String sn;
};