#include <ListItem.h>
#include <Bitmap.h>
#include <TranslationUtils.h>
#include <StringView.h>
#include <Path.h>
#include <Roster.h>
#include <Alert.h>
#include <Application.h>

#include <stdio.h>

#include "gamepiece.h"
#include "window.h"

#define window ((GameBoard *)(Window()))

BBitmap *get_bitmap(int orient);

const uint8 ship_length[] = {2,3,3,4,5};
const char ship_signal[] = {'D','S','C','B','A'};
const char *ship_name[] = {"destroyer","submarine","cruiser","battleship","carrier"};
char us[255];
		
square::square(int board,int x, int y) : BView(BRect(x*20,y*20,(x*20)+20,(y*20)+20),NULL,B_FOLLOW_LEFT | B_FOLLOW_TOP,B_WILL_DRAW | B_FRAME_EVENTS | B_FULL_UPDATE_ON_RESIZE) {
	SetViewColor(98,109,188);
	SetLowColor(98,109,188);
	ship = none;
	this->x = x;
	this->y = y;
	board_type = board;
	hit = 0;
	place = 0;
	pos = 0;
}
void square::SetShip(ship_type ship, int position, int direction) {
	this->ship = ship;
	place = position;
	pos = direction;
}
		
void square::Draw(BRect updateRect) {
	//--picts c/o Jason Matthews
	SetLowColor(ViewColor());
	if (ship != none) {
		int offset = ((place == (ship_length[ship] - 1)) ? (21*4) : (21*place));
		BRect src_rect = get_bitmap(pos)->Bounds();
		switch (pos) {
			case 0:
				src_rect.left += offset;
				src_rect.right = src_rect.left + 18;
				break;
			case 1:
				src_rect.top += offset;
				src_rect.bottom = src_rect.top + 18;
				break;
			case 2:
				src_rect.right -= offset;
				src_rect.left = src_rect.right - 18;
				break;
			case 3:
				src_rect.bottom -= offset;
				src_rect.top = src_rect.bottom - 18;
				break;
		}				
		BRect dest = Bounds();
		dest.InsetBy(1,1);
		DrawBitmap(get_bitmap(pos),src_rect,dest);
	}
	if (hit == 1)
		SetHighColor(255,0,0);
	if (hit == -1)
		SetHighColor(0,255,0);
	if (hit != 0)
		FillArc(BRect(7,7,13,13),0,360);
	SetHighColor(0,0,0);
	StrokeRect(Bounds());
}


void square::MouseDown(BPoint where) {
	if ((board_type == 2) && (hit == 0) && (our_turn == true)) {
		muscle::MessageRef ref = muscle::GetMessageFromPool(HIT_SQUARE);
		ref()->AddInt8("x",x);
		ref()->AddInt8("y",y);
		((GameBoard *)(Window()))->SendMessageToPartner(ref);
		window->status->SetText("Duck and Cover!");
		our_turn = false;
	} else if ((((GameBoard *)(Window()))->placing != none) && (board_type == 1)) {
		if (ship != none)
			return;
		
		if (((GameBoard *)(Window()))->lastx == -1) {
			((GameBoard *)(Window()))->lastx = x;
			((GameBoard *)(Window()))->lasty = y;
			ship = ((GameBoard *)(Window()))->placing;
			Invalidate();
		} else {
			if (abs(x-((GameBoard *)(Window()))->lastx) > abs(y-((GameBoard *)(Window()))->lasty)) {
				int inc = (x-((GameBoard *)(Window()))->lastx > 0) ? 1 : -1;
				int temp_lastx = window->lastx;
				if (window->lastx + (inc*ship_length[window->placing]) < 0)
					temp_lastx = ship_length[window->placing];
				if (window->lastx + (inc*ship_length[window->placing]) > 10)
					temp_lastx = 10 - ship_length[window->placing];
				int workx = temp_lastx+inc;
				for (;abs(workx-window->lastx) < ship_length[window->placing]; workx += inc) {
					if (window->squares[0][workx][window->lasty]->ship != none)
						return;
				}
				window->lastx = temp_lastx;
				workx = window->lastx;
				for (;abs(workx-window->lastx) < ship_length[window->placing]; workx += inc) {
					window->squares[0][workx][window->lasty]->SetShip(window->placing,abs(workx-window->lastx),(inc == -1) ? 2 : 0);
					window->squares[0][workx][window->lasty]->Invalidate();
				}
			} else {
				int inc = (y-((GameBoard *)(Window()))->lasty > 0) ? 1 : -1;
				int temp_lasty = window->lasty;
				if (window->lasty + (inc*ship_length[window->placing]) < 0)
					temp_lasty = ship_length[window->placing];
				if (window->lasty + (inc*ship_length[window->placing]) > 10)
					temp_lasty = 10 - ship_length[window->placing];
				int worky = temp_lasty+inc;
				for (;abs(worky-window->lasty) < ship_length[window->placing]; worky += inc) {
					if (window->squares[0][window->lastx][worky]->ship != none)
						return;
				}
				window->lasty = temp_lasty;
				worky = window->lasty;
				for (;abs(worky-window->lasty) < ship_length[window->placing]; worky += inc) {
					window->squares[0][window->lastx][worky]->SetShip(window->placing,abs(worky-window->lasty),(inc == -1) ? 3 : 1);
					window->squares[0][window->lastx][worky]->Invalidate();
				}
				
			}
			if (window->placing < carrier) {
				window->placing = ship_type(int(window->placing) + 1);
				char message[255];
				window->lastx = -1;
				window->lasty = -1;
				sprintf(message,"Please place %s...",ship_name[window->placing]);
				window->status->SetText(message);
			} else {
				window->placing = none;
				window->SendMessageToPartner(muscle::GetMessageFromPool(PLAY_BALL));
				if (!our_turn) {
					muscle::String status_msg;
					status_msg << "Waiting for " << window->opponent_name << " to finish placing ships...";
					window->status->SetText(status_msg());
				} else {
					window->status->SetText("Fire at will!");
				}
			}
		}		
	}
}

void square::Hit() {
	if (board_type == 1) {
		hit = (ship != none) ? 1 : -1;
		muscle::MessageRef ref = muscle::GetMessageFromPool(HIT_RESULT);
		ref()->AddInt8("x",x);
		ref()->AddInt8("y",y);
		ref()->AddBool("result",(hit == 1));
		((GameBoard *)(Window()))->SendMessageToPartner(ref);
		if (ship != none)
			((GameBoard *)(Window()))->Hit(ship);
	}
	Invalidate();
}

BBitmap *get_bitmap(int orient) {
	static BBitmap *picts[4] = {NULL,NULL,NULL,NULL};
	static BPath dir;
	char *names[4] = {"right","down","left","up"};
	if (picts[orient] == NULL) {
		if (dir.InitCheck() != B_OK) {
			app_info info;
			be_app->GetAppInfo(&info);
			BPath path(&(info.ref));
			path.GetParent(&dir);
			dir.Append("img");
		}
		BPath file(dir.Path(),names[orient]);
		picts[orient] = BTranslationUtils::GetBitmapFile(file.Path());
		if (picts[orient] == NULL)
			(new BAlert("ck","picts[orient] == NULL","OK"))->Go();
	}
	return picts[orient];
}
