#include <ListItem.h>
#include <Messenger.h>
#include <TextControl.h>
#include <ListView.h>
#include <ScrollView.h>
#include <StringView.h>
#include <String.h>
#include <Roster.h>
#include <Application.h>
#include <Alert.h>

#include "system/MessageTransceiverThread.h"
#include "reflector/StorageReflectConstants.h"
#include "util/String.h"
#include "besupport/BThread.h"

#include "gamepiece.h"
#include "window.h"

bool our_turn = false;

class Player : public BStringItem {
	public:
		Player(const char *text,int32 id) : BStringItem(text), id_num(id) {}
		
		int32 id_num;
};

GameBoard::GameBoard(const char *IP) : 
  BWindow(BRect(200,100,590,600),"Battleship",B_TITLED_WINDOW,B_NOT_RESIZABLE | B_NOT_ZOOMABLE),
  GameSession(IP,2960,us),
  lastx(-1),
  lasty(-1),
  placing(none),
  partner_ready(false),
  connected(false),
  host(IP) {
	BView *opponent, *me;
	opponent = new BView(BRect(25,25,225,225),"opponent",B_FOLLOW_LEFT | B_FOLLOW_TOP,0);
	me = new BView(BRect(25,265,225,465),"me",B_FOLLOW_LEFT | B_FOLLOW_TOP,0);
	name_box = new BTextControl(BRect(230,25,340,45),"name_box","Name: ",us,new BMessage('name'));
	name_box->SetDivider(be_plain_font->StringWidth("Name: "));
	AddChild(name_box);
	players = new BListView(BRect(230,50,340,448),"playerslist");
	playersbox = new BScrollView("playersbox",players,B_FOLLOW_LEFT | B_FOLLOW_TOP, 0, false, true);
	players->SetInvocationMessage(new BMessage('chos'));
	AddChild(playersbox);
	for (uint8 x = 0; x < 10; x++) {
		for (uint8 y = 0; y < 10; y++) {
			squares[0][x][y] = new square(1,x,y);
			me->AddChild(squares[0][x][y]);
		}
	}
	for (uint8 x = 0; x < 10; x++) {
		for (uint8 y = 0; y < 10; y++) {
			squares[1][x][y] = new square(2,x,y);
			opponent->AddChild(squares[1][x][y]);
		}
	}
	for (int i = 0; i < 5; i++)
		ship_count[i] = 0;
	muscle::String stat;
	stat << "Connecting to " << IP << "...";
	status = new BStringView(BRect(5,230,245,245),"status",stat());
	chat_display = new BStringView(BRect(5,245,245,260),"chat_display","");
	chat = new BTextControl(BRect(10,470,240,487),"chat","Chat: ","",new BMessage('sdct'));
	chat->SetDivider(be_plain_font->StringWidth("Chat: "));
	AddChild(status);
	AddChild(chat_display);
	AddChild(opponent);
	AddChild(me);
	Show();
	accept = false;
	ships_sunk = 0;
}

void GameBoard::Hit(ship_type ship) {
	ship_count[ship]++;
	if (ship_count[ship] >= ship_length[ship]) {
		muscle::MessageRef msg = muscle::GetMessageFromPool(SHIP_SUNK);
		msg()->AddInt8("ship",ship);
		SendMessageToPartner(msg);
	}
}

void GameBoard::PartnerGone() {
	GameSession::PartnerGone();
	if (connected)
		(new BAlert("evilone","Your opponent has disconnected. That's not very nice of him.","Bad doggy!"))->Go();
	
	our_turn = false;
	connected = false;
	AddChild(playersbox);
	AddChild(name_box);
	Lock();
	chat->RemoveSelf();
	chat_display->SetText("");
	ResizeTo(360,500);
	lastx = -1;
	lasty = -1;
	muscle::String foo;
	foo << "Connected to " << host;
	status->SetText(foo());
	Unlock();
	opponent_name = "";
}

void GameBoard::NewUserLoggedOn(int32 id, const char *name) {
	Lock();
	players->AddItem(new Player(name,id));
	Unlock();
}

void GameBoard::UserLoggedOff(int32 id, const char *name) {
	Lock();
	printf("User gone: %s (%d)\n",name,id);
	for (int32 i = 0; i < players->CountItems(); i++) {
		if (((Player *)(players->ItemAt(i)))->id_num == id)
			players->RemoveItem(i);
	}
	Unlock();
}

void GameBoard::ServerDisconnected() {
	(new BAlert("ah, shit","Either you've been booted off the server, or it died. Goodbye, world!","Goodbye!"))->Go();
	be_app->Quit();
	GameSession::ServerDisconnected();
}

void GameBoard::ServerConnectDone() {
	GameSession::ServerConnectDone();
	muscle::String stat;
	stat << "Connected to " << host;
	Lock();
	status->SetText(stat());
	Unlock();
}	

bool GameBoard::AllowConnectFrom(const char *user_name, int32 id) {
	if (!connected) {
		accept = true;
		PrepareBoardForPlayTime(user_name);
		return true;
	}
	return false;
}

void GameBoard::MessageReceivedFromPartner(muscle::MessageRef msg) {
	int8 x,y;
	switch (msg()->what) {
		case HIT_SQUARE:
			Lock();
			msg()->FindInt8("x",0,&x);
			msg()->FindInt8("y",0,&y);
			squares[0][x][y]->Hit();
			Unlock();
			break;
		case HIT_RESULT: {
			bool result;
			
			msg()->FindInt8("x",0,&x);
			msg()->FindInt8("y",0,&y);
			msg()->FindBool("result",0,&result);
			Lock();
			squares[1][x][y]->hit = (result) ? 1 : -1;
			squares[1][x][y]->Invalidate();
			Unlock();
			
			SendMessageToPartner(muscle::GetMessageFromPool(YOUR_TURN));
			break; }
		case SHIP_SUNK: {
			char message[255];
			msg()->FindInt8("ship",0,&x);
			sprintf(message,"You have sunk the enemy's %s!",ship_name[x]);
			(new BAlert("sunk",message,"Yay!"))->Go();
			ships_sunk++;
			if (ships_sunk >= 5) {
				SendMessageToPartner(muscle::GetMessageFromPool(YOU_LOSE));
				(new BAlert("win","You win!","Hooray!"))->Go();
				connected = false;
				//---EndGame() calls PartnerGone() for us
				EndGame();
			}
			break; }
		case YOU_LOSE: {
			(new BAlert("win","You lose!","Phooey"))->Go();
			connected = false;
			//------The other has terminated our connection, so we don't need to do it
			//---- Also, we get our GOOD_BYE message here, so we don't need to do anything
			break; }
		case PLAY_BALL:
			partner_ready = true;
			if (accept)
				our_turn = true;
			if ((placing == none) && (!accept)) {
				Lock();
				status->SetText("Duck and Cover!");
				Unlock();
			}
			if (!((placing == none) && (accept)))
				break;
		case YOUR_TURN:
			our_turn = true;
			Lock();
			status->SetText("Fire at will!");
			Unlock();
			break;
		case 'chat':
			{
				muscle::String string;
				msg()->FindString("chat",string);
				string = opponent_name + ": " + string;
				Lock();
				chat_display->SetText(string());
				Unlock();
			}
			break;
	}
	
	GameSession::MessageReceivedFromPartner(msg);
}

const char * GameBoard::GetTypeName() const {
	return "Battleship";
}

void GameBoard::PrepareBoardForPlayTime(const char *user) {
	if (find_thread(NULL) != BLooper::Thread())
		Lock();
	
	for (uint8 x = 0; x < 10; x++) {
		for (uint8 y = 0; y < 10; y++) {
			squares[0][x][y]->SetShip(none,0,0);
			squares[0][x][y]->hit = 0;
			squares[0][x][y]->SetHighColor(squares[0][x][y]->ViewColor());
			squares[0][x][y]->FillRect(squares[0][x][y]->Bounds());
			squares[0][x][y]->Draw(squares[0][x][y]->Bounds());
		}
	}
	for (uint8 x = 0; x < 10; x++) {
		for (uint8 y = 0; y < 10; y++) {
			squares[1][x][y]->SetShip(none,0,0);
			squares[1][x][y]->hit = 0;
			squares[1][x][y]->SetHighColor(squares[1][x][y]->ViewColor());
			squares[1][x][y]->FillRect(squares[1][x][y]->Bounds());
			squares[1][x][y]->Draw(squares[1][x][y]->Bounds());
		}
	}
	char text[255];
	opponent_name = user;
	sprintf(text,"%s Connected. Please place destroyer...",user);
	status->SetText(text);
	placing = destroyer;
	ship_count[0] = 0;
	ship_count[1] = 0;
	ship_count[2] = 0;
	ship_count[3] = 0;
	ship_count[4] = 0;
	
	ships_sunk = 0;
	ResizeTo(250,500);
	playersbox->RemoveSelf();
	name_box->RemoveSelf();
	AddChild(chat);
	connected = true;
	
	if (find_thread(NULL) != BLooper::Thread())
		Unlock();
}

void GameBoard::MessageReceived(BMessage *msg) {
	switch (msg->what) {
        case 'chos': {
        	if (players->CurrentSelection() < 0)
        		break;
        	const char *user = ((BStringItem *)(players->ItemAt(players->CurrentSelection())))->Text();
        		
        	if (ConnectTo(((Player *)(players->ItemAt(players->CurrentSelection())))->id_num)) {
        		accept = false;
        		PrepareBoardForPlayTime(user);
			} else {
				char text[255];
				sprintf(text,"%s is too good to play with you (probably already playing)",user);
				(new BAlert("dang",text,"OK"))->Go();
			}
			break; }
		case 'name': {
			strcpy(us,name_box->Text());
			ChangeName(us);
			app_info info;
			be_app->GetAppInfo(&info);
			BNode(&(info.ref)).WriteAttr("user_name",'CSTR',0,us,strlen(us)+1);
			break; }
		case 'sdct':
			{
				muscle::MessageRef ref = muscle::GetMessageFromPool('chat');
				ref()->AddString("chat",chat->Text());
				chat->SetText("");
				SendMessageToPartner(ref);
			}
			break;
	}
}

bool GameBoard::QuitRequested(void) {
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}
